type bool =
  | True
  | False

type nat =
  | O
  | S of nat

type 'a option =
  | Some of 'a
  | None

type ('a, 'b) prod =
  | Pair of 'a * 'b

(** val fst : ('a1, 'a2) prod -> 'a1 **)

let fst = function
  | Pair (x, y) -> x

(** val snd : ('a1, 'a2) prod -> 'a2 **)

let snd = function
  | Pair (x, y) -> y

type comparison =
  | Eq
  | Lt
  | Gt

type 'a sig0 = 'a
  (* singleton inductive, whose constructor was exist *)

type sumbool =
  | Left
  | Right

type 'a list =
  | Nil
  | Cons of 'a * 'a list

(** val length : 'a1 list -> nat **)

let rec length = function
  | Nil -> O
  | Cons (a, m) -> S (length m)

(** val app : 'a1 list -> 'a1 list -> 'a1 list **)

let rec app l m =
  match l with
    | Nil -> m
    | Cons (a, l1) -> Cons (a, (app l1 m))

(** val in_dec : ('a1 -> 'a1 -> sumbool) -> 'a1 -> 'a1 list -> sumbool **)

let rec in_dec h a = function
  | Nil -> Right
  | Cons (a0, l0) ->
      (match h a0 a with
         | Left -> Left
         | Right -> in_dec h a l0)

module Coq__1 = struct 
 (** val map : ('a1 -> 'a2) -> 'a1 list -> 'a2 list **)
 
 let rec map f = function
   | Nil -> Nil
   | Cons (a, t0) -> Cons ((f a), (map f t0))
end
let map = Coq__1.map


(** val fold_left : ('a1 -> 'a2 -> 'a1) -> 'a2 list -> 'a1 -> 'a1 **)

let rec fold_left f l a0 =
  match l with
    | Nil -> a0
    | Cons (b, t0) -> fold_left f t0 (f a0 b)

(** val existsb : ('a1 -> bool) -> 'a1 list -> bool **)

let rec existsb f = function
  | Nil -> False
  | Cons (a, l0) -> (match f a with
                       | True -> True
                       | False -> existsb f l0)

(** val forallb : ('a1 -> bool) -> 'a1 list -> bool **)

let rec forallb f = function
  | Nil -> True
  | Cons (a, l0) ->
      (match f a with
         | True -> forallb f l0
         | False -> False)

(** val filter : ('a1 -> bool) -> 'a1 list -> 'a1 list **)

let rec filter f = function
  | Nil -> Nil
  | Cons (x, l0) ->
      (match f x with
         | True -> Cons (x, (filter f l0))
         | False -> filter f l0)

type positive =
  | XI of positive
  | XO of positive
  | XH

(** val pcompare : positive -> positive -> comparison -> comparison **)

let rec pcompare x y r =
  match x with
    | XI p ->
        (match y with
           | XI q -> pcompare p q r
           | XO q -> pcompare p q Gt
           | XH -> Gt)
    | XO p ->
        (match y with
           | XI q -> pcompare p q Lt
           | XO q -> pcompare p q r
           | XH -> Gt)
    | XH -> (match y with
               | XH -> r
               | _ -> Lt)

(** val peq : positive -> positive -> sumbool **)

let peq x y =
  match pcompare x y Eq with
    | Eq -> Left
    | _ -> Right

(** val peq0 : positive -> positive -> sumbool **)

let peq0 x y =
  peq x y

(** val list_prod :
    ('a1 -> 'a2 -> 'a3) -> 'a1 list -> 'a2 list -> 'a3 list **)

let rec list_prod f l l' =
  match l with
    | Nil -> Nil
    | Cons (x, t0) -> app (map (f x) l') (list_prod f t0 l')

(** val list_combine :
    ('a1 -> 'a2 -> 'a3) -> 'a1 list -> 'a2 list -> 'a3 list **)

let rec list_combine f l l' =
  match l with
    | Nil -> Nil
    | Cons (a, tl) ->
        (match l' with
           | Nil -> Nil
           | Cons (b, tl') -> Cons ((f a b), (list_combine f tl tl')))

type ident = positive

(** val id_eq : positive -> positive -> sumbool **)

let id_eq x y =
  peq0 x y

type term =
  | Fun of ident * term list
  | Var of ident
  | State of ident

(** val term_rect' :
    (ident -> 'a1) -> (ident -> 'a1) -> (ident -> term list -> 'a2 -> 'a1) ->
    'a2 -> (term -> term list -> 'a1 -> 'a2 -> 'a2) -> term -> 'a1 **)

let rec term_rect' h0 h1 h2 h3 h4 = function
  | Fun (p, l) ->
      h2 p l
        (let rec f0 = function
           | Nil -> h3
           | Cons (t', lt') ->
               h4 t' lt' (term_rect' h0 h1 h2 h3 h4 t') (f0 lt')
         in f0 l)
  | Var x -> h0 x
  | State q -> h1 q

(** val term_eq_dec : term -> term -> sumbool **)

let term_eq_dec u v =
  term_rect' (fun x v0 -> match v0 with
                            | Var i -> id_eq x i
                            | _ -> Right) (fun q v0 ->
    match v0 with
      | State i -> id_eq q i
      | _ -> Right) (fun p l iHu v0 ->
    match v0 with
      | Fun (i, l0) ->
          (match id_eq p i with
             | Left -> iHu l0
             | Right -> Right)
      | _ -> Right) (fun k ->
    match k with
      | Nil -> Left
      | Cons (t0, k0) -> Right) (fun u0 lt iHu iHu0 k ->
    match k with
      | Nil -> Right
      | Cons (t0, k0) ->
          (match iHu t0 with
             | Left -> iHu0 k0
             | Right -> Right)) u v

type l_rule =
  | Lrule of term * term

type l_rules = l_rule list

type substitution = ident -> term option

(** val add_subst : substitution -> ident -> term -> substitution option **)

let add_subst s x v =
  match s x with
    | Some w ->
        (match term_eq_dec v w with
           | Left -> Some s
           | Right -> None)
    | None -> Some (fun y ->
        match id_eq x y with
          | Left -> Some v
          | Right -> s y)

(** val subst : substitution -> term -> term **)

let rec subst s t0 = match t0 with
  | Fun (f, l) -> Fun (f, (map (subst s) l))
  | Var x -> (match s x with
                | Some v -> v
                | None -> t0)
  | State i -> t0

module type TREE = 
 sig 
  type elt 
  
  val elt_eq : elt -> elt -> sumbool
  
  type 'x t 
  
  val eq : ('a1 -> 'a1 -> sumbool) -> 'a1 t -> 'a1 t -> sumbool
  
  val empty : 'a1 t
  
  val get : elt -> 'a1 t -> 'a1 option
  
  val set : elt -> 'a1 -> 'a1 t -> 'a1 t
  
  val remove : elt -> 'a1 t -> 'a1 t
  
  val is_empty : 'a1 t -> sumbool
  
  val map : (elt -> 'a1 -> 'a2) -> 'a1 t -> 'a2 t
  
  val mapi : (elt -> 'a1 -> 'a2 option) -> 'a1 t -> 'a2 t
  
  val combine :
    ('a1 option -> 'a1 option -> 'a1 option) -> 'a1 t -> 'a1 t -> 'a1 t
  
  val elements : 'a1 t -> (elt, 'a1) prod list
  
  val fold : ('a2 -> elt -> 'a1 -> 'a2) -> 'a1 t -> 'a2 -> 'a2
  
  val left : elt -> elt
  
  val right : elt -> elt
  
  val join : 'a1 t -> 'a1 t -> 'a1 t
 end

module PTree = 
 struct 
  type elt = positive
  
  (** val elt_eq : positive -> positive -> sumbool **)
  
  let elt_eq x y =
    peq x y
  
  type 'a tree =
    | Leaf
    | Node of 'a tree * 'a option * 'a tree
  
  (** val tree_rect :
      'a2 -> ('a1 tree -> 'a2 -> 'a1 option -> 'a1 tree -> 'a2 -> 'a2) -> 'a1
      tree -> 'a2 **)
  
  let rec tree_rect f f0 = function
    | Leaf -> f
    | Node (t1, o, t2) -> f0 t1 (tree_rect f f0 t1) o t2 (tree_rect f f0 t2)
  
  (** val tree_rec :
      'a2 -> ('a1 tree -> 'a2 -> 'a1 option -> 'a1 tree -> 'a2 -> 'a2) -> 'a1
      tree -> 'a2 **)
  
  let rec tree_rec f f0 = function
    | Leaf -> f
    | Node (t1, o, t2) -> f0 t1 (tree_rec f f0 t1) o t2 (tree_rec f f0 t2)
  
  type 'a t = 'a tree
  
  (** val eq : ('a1 -> 'a1 -> sumbool) -> 'a1 t -> 'a1 t -> sumbool **)
  
  let rec eq eqA a b =
    match a with
      | Leaf -> (match b with
                   | Leaf -> Left
                   | Node (t0, o, t1) -> Right)
      | Node (t0, o, t1) ->
          (match b with
             | Leaf -> Right
             | Node (t2, o0, t3) ->
                 (match eq eqA t0 t2 with
                    | Left ->
                        (match match o with
                                 | Some x ->
                                     (match o0 with
                                        | Some a1 -> eqA x a1
                                        | None -> Right)
                                 | None ->
                                     (match o0 with
                                        | Some a0 -> Right
                                        | None -> Left) with
                           | Left -> eq eqA t1 t3
                           | Right -> Right)
                    | Right -> Right))
  
  (** val empty : 'a1 t **)
  
  let empty =
    Leaf
  
  (** val is_empty : 'a1 t -> sumbool **)
  
  let is_empty = function
    | Leaf -> Left
    | Node (x1, o, x2) -> Right
  
  (** val get : positive -> 'a1 t -> 'a1 option **)
  
  let rec get i = function
    | Leaf -> None
    | Node (l, o, r) ->
        (match i with
           | XI ii -> get ii r
           | XO ii -> get ii l
           | XH -> o)
  
  (** val set : positive -> 'a1 -> 'a1 t -> 'a1 t **)
  
  let rec set i v = function
    | Leaf ->
        (match i with
           | XI ii -> Node (Leaf, None, (set ii v Leaf))
           | XO ii -> Node ((set ii v Leaf), None, Leaf)
           | XH -> Node (Leaf, (Some v), Leaf))
    | Node (l, o, r) ->
        (match i with
           | XI ii -> Node (l, o, (set ii v r))
           | XO ii -> Node ((set ii v l), o, r)
           | XH -> Node (l, (Some v), r))
  
  (** val remove : positive -> 'a1 t -> 'a1 t **)
  
  let rec remove i m =
    match i with
      | XI ii ->
          (match m with
             | Leaf -> Leaf
             | Node (l, o, r) ->
                 (match l with
                    | Leaf ->
                        (match o with
                           | Some a -> Node (l, o, (remove ii r))
                           | None ->
                               (match remove ii r with
                                  | Leaf -> Leaf
                                  | Node (t0, o0, t1) -> Node (Leaf, None,
                                      (Node (t0, o0, t1)))))
                    | Node (t0, o0, t1) -> Node (l, o, (remove ii r))))
      | XO ii ->
          (match m with
             | Leaf -> Leaf
             | Node (l, o, r) ->
                 (match o with
                    | Some a -> Node ((remove ii l), o, r)
                    | None ->
                        (match r with
                           | Leaf ->
                               (match remove ii l with
                                  | Leaf -> Leaf
                                  | Node (t0, o0, t1) -> Node ((Node (t0, o0,
                                      t1)), None, Leaf))
                           | Node (t0, o0, t1) -> Node ((remove ii l), o, r))))
      | XH ->
          (match m with
             | Leaf -> Leaf
             | Node (l, o, r) ->
                 (match l with
                    | Leaf ->
                        (match r with
                           | Leaf -> Leaf
                           | Node (t0, o0, t1) -> Node (l, None, r))
                    | Node (t0, o0, t1) -> Node (l, None, r)))
  
  (** val append : positive -> positive -> positive **)
  
  let rec append i j =
    match i with
      | XI ii -> XI (append ii j)
      | XO ii -> XO (append ii j)
      | XH -> j
  
  (** val xmap : (positive -> 'a1 -> 'a2) -> 'a1 t -> positive -> 'a2 t **)
  
  let rec xmap f m i =
    match m with
      | Leaf -> Leaf
      | Node (l, o, r) -> Node ((xmap f l (append i (XO XH))),
          (match o with
             | Some y -> Some (f i y)
             | None -> None), (xmap f r (append i (XI XH))))
  
  (** val map : (positive -> 'a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let map f m =
    xmap f m XH
  
  (** val xmapi :
      (positive -> 'a1 -> 'a2 option) -> 'a1 t -> positive -> 'a2 t **)
  
  let rec xmapi f m i =
    match m with
      | Leaf -> Leaf
      | Node (l, o, r) -> Node ((xmapi f l (append i (XO XH))),
          (match o with
             | Some x -> f i x
             | None -> None), (xmapi f r (append i (XI XH))))
  
  (** val mapi : (positive -> 'a1 -> 'a2 option) -> 'a1 t -> 'a2 t **)
  
  let mapi f m =
    xmapi f m XH
  
  (** val xcombine_l :
      ('a1 option -> 'a1 option -> 'a1 option) -> 'a1 t -> 'a1 t **)
  
  let rec xcombine_l f = function
    | Leaf -> Leaf
    | Node (l, o, r) -> Node ((xcombine_l f l), (f o None), (xcombine_l f r))
  
  (** val xcombine_r :
      ('a1 option -> 'a1 option -> 'a1 option) -> 'a1 t -> 'a1 t **)
  
  let rec xcombine_r f = function
    | Leaf -> Leaf
    | Node (l, o, r) -> Node ((xcombine_r f l), (f None o), (xcombine_r f r))
  
  (** val combine :
      ('a1 option -> 'a1 option -> 'a1 option) -> 'a1 t -> 'a1 t -> 'a1 t **)
  
  let rec combine f m1 m2 =
    match m1 with
      | Leaf -> xcombine_r f m2
      | Node (l1, o1, r1) ->
          (match m2 with
             | Leaf -> xcombine_l f m1
             | Node (l2, o2, r2) -> Node ((combine f l1 l2), 
                 (f o1 o2), (combine f r1 r2)))
  
  (** val xelements : 'a1 t -> positive -> (positive, 'a1) prod list **)
  
  let rec xelements m i =
    match m with
      | Leaf -> Nil
      | Node (l, o, r) ->
          (match o with
             | Some x ->
                 app (xelements l (append i (XO XH))) (Cons ((Pair (i, x)),
                   (xelements r (append i (XI XH)))))
             | None ->
                 app (xelements l (append i (XO XH)))
                   (xelements r (append i (XI XH))))
  
  (** val elements : 'a1 t -> (positive, 'a1) prod list **)
  
  let elements m =
    xelements m XH
  
  (** val xget : positive -> positive -> 'a1 t -> 'a1 option **)
  
  let rec xget i j m =
    match i with
      | XI ii ->
          (match j with
             | XI jj -> xget ii jj m
             | XO p -> None
             | XH -> get i m)
      | XO ii ->
          (match j with
             | XI p -> None
             | XO jj -> xget ii jj m
             | XH -> get i m)
      | XH -> (match j with
                 | XH -> get i m
                 | _ -> None)
  
  (** val xkeys : 'a1 t -> positive -> positive list **)
  
  let xkeys m i =
    Coq__1.map (fun x -> fst x) (xelements m i)
  
  (** val fold : ('a2 -> positive -> 'a1 -> 'a2) -> 'a1 t -> 'a2 -> 'a2 **)
  
  let fold f tr v =
    fold_left (fun a p -> f a (fst p) (snd p)) (elements tr) v
  
  (** val left : positive -> positive **)
  
  let left p =
    XO p
  
  (** val right : positive -> positive **)
  
  let right p =
    XI p
  
  (** val join : 'a1 t -> 'a1 t -> 'a1 tree **)
  
  let join t1 t2 =
    Node (t1, None, t2)
 end

module type SET_LIST = 
 sig 
  type coq_A 
  
  type t 
  
  val empty : t
  
  val inl : coq_A list -> t -> sumbool
  
  val is_empty : t -> sumbool
  
  val lists : t -> coq_A list list
  
  val add : coq_A list -> t -> t
  
  val merge : t -> t -> t
  
  val prod : (coq_A -> coq_A -> coq_A) -> t -> t -> t
  
  val fold : ('a1 -> coq_A list -> 'a1) -> t -> 'a1 -> 'a1
  
  val map : (coq_A -> coq_A) -> t -> t
  
  val filter : (coq_A list -> bool) -> t -> t
 end

module type KEY = 
 sig 
  type coq_A 
  
  val eq : coq_A -> coq_A -> sumbool
 end

module type STATE = 
 KEY

module State = 
 struct 
  type coq_A = positive
  
  (** val eq : positive -> positive -> sumbool **)
  
  let eq x y =
    peq x y
 end

module MakeSetList = 
 functor (St:STATE) ->
 struct 
  type coq_A = St.coq_A
  
  (** val eqdec : St.coq_A -> St.coq_A -> sumbool **)
  
  let eqdec x y =
    St.eq x y
  
  type t = coq_A list list
  
  (** val empty : coq_A list list **)
  
  let empty =
    Nil
  
  (** val list_eqdec : coq_A list -> coq_A list -> sumbool **)
  
  let rec list_eqdec l l0 =
    match l with
      | Nil -> (match l0 with
                  | Nil -> Left
                  | Cons (a, l1) -> Right)
      | Cons (a, l1) ->
          (match l0 with
             | Nil -> Right
             | Cons (a0, l2) ->
                 (match eqdec a a0 with
                    | Left -> list_eqdec l1 l2
                    | Right -> Right))
  
  (** val inl : coq_A list -> t -> sumbool **)
  
  let rec inl l = function
    | Nil -> Right
    | Cons (a, l0) ->
        (match list_eqdec a l with
           | Left -> Left
           | Right -> inl l l0)
  
  (** val is_empty : t -> sumbool **)
  
  let is_empty = function
    | Nil -> Left
    | Cons (l, e0) -> Right
  
  (** val lists : t -> t **)
  
  let lists m =
    m
  
  (** val add : coq_A list -> t -> t **)
  
  let add l m =
    Cons (l, m)
  
  (** val merge : t -> t -> t **)
  
  let rec merge t1 t2 =
    match t1 with
      | Nil -> t2
      | Cons (l, t0) ->
          (match inl l t2 with
             | Left -> merge t0 t2
             | Right -> add l (merge t0 t2))
  
  (** val prod : (coq_A -> coq_A -> coq_A) -> t -> t -> t **)
  
  let prod f t1 t2 =
    list_prod (fun x x0 -> list_combine f x x0) t1 t2
  
  (** val fold : ('a1 -> coq_A list -> 'a1) -> t -> 'a1 -> 'a1 **)
  
  let fold f m b0 =
    fold_left f m b0
  
  (** val map : (coq_A -> coq_A) -> t -> t **)
  
  let rec map f = function
    | Nil -> empty
    | Cons (l, m') -> add (Coq__1.map f l) (map f m')
  
  (** val filter :
      (coq_A list -> bool) -> coq_A list list -> coq_A list list **)
  
  let filter f l =
    filter f l
 end

module Make = 
 functor (Tree:TREE with type elt=
 positive) ->
 functor (SetList:SET_LIST with type coq_A=
 positive) ->
 struct 
  type t = SetList.t Tree.t Tree.t
  
  (** val empty : SetList.t Tree.t Tree.t **)
  
  let empty =
    Tree.empty
  
  (** val get0 : positive -> t -> SetList.t Tree.t **)
  
  let get0 i d0 =
    match Tree.get i d0 with
      | Some x -> x
      | None -> Tree.empty
  
  (** val get1 : positive -> SetList.t Tree.t -> SetList.t **)
  
  let get1 i m =
    match Tree.get i m with
      | Some x -> x
      | None -> SetList.empty
  
  (** val get : positive -> positive -> t -> SetList.t **)
  
  let get i j d0 =
    match Tree.get j (get0 i d0) with
      | Some x -> x
      | None -> SetList.empty
  
  (** val symbol_list : positive -> t -> positive list **)
  
  let symbol_list i d0 =
    Tree.fold (fun l k v -> Cons (k, l)) (get0 i d0) Nil
  
  (** val dom : t -> positive list **)
  
  let dom d0 =
    Tree.fold (fun l k v -> Cons (k, l)) d0 Nil
  
  (** val add : positive -> positive -> positive list -> t -> t **)
  
  let add i j l d0 =
    Tree.set i (Tree.set j (SetList.add l (get i j d0)) (get0 i d0)) d0
  
  (** val map : (positive -> positive) -> t -> t **)
  
  let map f d0 =
    Tree.map (fun x x0 -> Tree.map (fun x1 -> SetList.map f) x0) d0
  
  (** val st_forall : (positive -> bool) -> t -> bool **)
  
  let st_forall f d0 =
    Tree.fold (fun b k x -> match f k with
                              | True -> b
                              | False -> False) d0 True
  
  (** val fold :
      ('a1 -> positive -> positive -> positive list -> 'a1) -> t -> 'a1 ->
      'a1 **)
  
  let fold f d0 a =
    Tree.fold (fun b q m ->
      Tree.fold (fun c s sl -> SetList.fold (fun e l -> f e q s l) sl c) m b)
      d0 a
  
  (** val left : positive -> Tree.elt **)
  
  let left i =
    Tree.left i
  
  (** val right : positive -> Tree.elt **)
  
  let right i =
    Tree.right i
  
  (** val join : t -> t -> t **)
  
  let join d1 d2 =
    Tree.join (map left d1) (map right d2)
 end

module SetList = MakeSetList(State)

module Delta = Make(PTree)(SetList)

type state = positive

type automaton = { qf : state list; d : Delta.t }

(** val qf : automaton -> state list **)

let qf x = x.qf

(** val d : automaton -> Delta.t **)

let d x = x.d

(** val match_conf : Delta.t -> positive -> term -> bool **)

let rec match_conf d0 q = function
  | Fun (f, l) ->
      let rec g = function
        | Nil -> False
        | Cons (lq, ll') ->
            (match let rec f0 lq0 = function
                     | Nil ->
                         (match lq0 with
                            | Nil -> True
                            | Cons (p, l0) -> False)
                     | Cons (c0, l') ->
                         (match lq0 with
                            | Nil -> False
                            | Cons (q0, ll'0) ->
                                (match match_conf d0 q0 c0 with
                                   | True -> f0 ll'0 l'
                                   | False -> False))
                   in f0 lq l with
               | True -> True
               | False -> g ll')
      in g (SetList.lists (Delta.get q f d0))
  | Var i -> False
  | State q' -> (match id_eq q q' with
                   | Left -> True
                   | Right -> False)

type env = (state, state) prod list

(** val states_dec :
    (state, state) prod -> (state, state) prod -> sumbool **)

let states_dec x y =
  let Pair (x1, x2) = x in
  let Pair (y1, y2) = y in
  (match id_eq x1 y1 with
     | Left -> id_eq x2 y2
     | Right -> Right)

(** val add_list_1 : env -> positive list -> positive list -> env **)

let add_list_1 d0 k l =
  fold_left (fun d1 x ->
    fold_left (fun d2 y -> Cons ((Pair (x, y)), d2)) l d1) k d0

(** val full_set : state list -> state list -> env **)

let full_set k l =
  add_list_1 Nil k l

(** val del : env -> (state, state) prod -> env **)

let rec del d0 e =
  match d0 with
    | Nil -> Nil
    | Cons (a, d') ->
        (match states_dec a e with
           | Left -> d'
           | Right -> Cons (a, (del d' e)))

(** val suppr : env -> env -> env **)

let rec suppr d0 e =
  fold_left del e d0

(** val mu : state list -> state list -> env -> nat **)

let mu k l e =
  length (suppr (full_set k l) e)

(** val mu0 : Delta.t -> Delta.t -> env -> nat **)

let mu0 d1 d2 e =
  mu (Delta.dom d1) (Delta.dom d2) e

(** val mu_defined : Delta.t -> Delta.t -> env -> nat **)

let mu_defined d1 d2 e =
  mu0 d1 d2 e

(** val all_nil_or_not' : Delta.t -> positive -> sumbool **)

let all_nil_or_not' d0 q =
  let rec f = function
    | Nil -> Left
    | Cons (a, l0) ->
        (match SetList.lists (Delta.get q a d0) with
           | Nil -> f l0
           | Cons (l1, l2) -> Right)
  in f (Delta.symbol_list q d0)

(** val istate_dec :
    Delta.t -> Delta.t -> env -> state -> state -> sumbool **)

let istate_dec d1 d2 e q q' =
  let rec f n e0 q0 q'0 =
    match n with
      | O ->
          (match in_dec id_eq q0 (Delta.dom d1) with
             | Left ->
                 (match in_dec id_eq q'0 (Delta.dom d2) with
                    | Left -> Left
                    | Right ->
                        (match in_dec states_dec (Pair (q0, q'0)) e0 with
                           | Left -> Left
                           | Right -> all_nil_or_not' d1 q0))
             | Right -> Left)
      | S n0 ->
          (match in_dec states_dec (Pair (q0, q'0)) e0 with
             | Left -> Left
             | Right ->
                 let rec f0 = function
                   | Nil -> Left
                   | Cons (a, l0) ->
                       (match f0 l0 with
                          | Left ->
                              let rec f1 = function
                                | Nil -> Left
                                | Cons (a0, l2) ->
                                    (match let rec f2 l3 c =
                                             match l3 with
                                               | Nil -> Right
                                               | Cons (
                                                  a1, l4) ->
                                                  (match 
                                                  let rec f3 l5 c' =
                                                    
                                                  match l5 with
                                                    | 
                                                  Nil ->
                                                  (match c' with
                                                    | 
                                                  Nil -> Left
                                                    | 
                                                  Cons (s, c'0) -> Right)
                                                    | 
                                                  Cons (
                                                  a2, l6) ->
                                                  (match c' with
                                                    | 
                                                  Nil -> Right
                                                    | 
                                                  Cons (
                                                  s, c'0) ->
                                                  (match 
                                                  f n0 (Cons ((Pair (q0,
                                                  q'0)), e0)) a2 s with
                                                    | 
                                                  Left -> f3 l6 c'0
                                                    | 
                                                  Right -> Right))
                                                  in f3 c a1 with
                                                    | 
                                                  Left -> Left
                                                    | 
                                                  Right -> f2 l4 c)
                                           in f2
                                                (SetList.lists
                                                  (Delta.get q'0 a d2)) a0 with
                                       | Left -> f1 l2
                                       | Right -> Right)
                              in f1 (SetList.lists (Delta.get q0 a d1))
                          | Right -> Right)
                 in f0 (Delta.symbol_list q0 d1))
  in f (mu_defined d1 d2 e) e q q'

(** val inclusion_dec : automaton -> automaton -> bool **)

let inclusion_dec a b =
  forallb (fun qa ->
    existsb (fun qb ->
      match istate_dec a.d b.d Nil qa qb with
        | Left -> True
        | Right -> False) b.qf) a.qf

type substitutions = substitution list

(** val matching_rec :
    Delta.t -> substitutions -> term -> positive -> substitutions **)

let rec matching_rec d0 ls c q =
  match c with
    | Fun (f, l) ->
        fold_left (fun ls' x ->
          app
            (let rec f0 ls0 lc lq =
               match lc with
                 | Nil ->
                     (match lq with
                        | Nil -> ls0
                        | Cons (p, l0) -> Nil)
                 | Cons (c0, l0) ->
                     (match lq with
                        | Nil -> Nil
                        | Cons (q0, l') ->
                            f0 (matching_rec d0 ls0 c0 q0) l0 l')
             in f0 ls l x) ls') (SetList.lists (Delta.get q f d0)) Nil
    | Var x ->
        fold_left (fun ls' s ->
          match add_subst s x (State q) with
            | Some s' -> Cons (s', ls')
            | None -> ls') ls Nil
    | State q' -> (match id_eq q q' with
                     | Left -> ls
                     | Right -> Nil)

(** val matching : Delta.t -> positive -> term -> substitutions **)

let matching d0 q c =
  matching_rec d0 (Cons ((fun x -> None), Nil)) c q

type substitutions0 = substitution list

type state0 = positive

(** val all_reduce : Delta.t -> state0 -> term -> substitutions0 -> bool **)

let all_reduce d0 q r ls =
  forallb (fun s -> match_conf d0 q (subst s r)) ls

(** val list_eq : state0 list -> state0 list -> bool **)

let rec list_eq k l =
  match k with
    | Nil -> (match l with
                | Nil -> True
                | Cons (s, l0) -> False)
    | Cons (a, k') ->
        (match l with
           | Nil -> False
           | Cons (b, l') ->
               (match id_eq a b with
                  | Left -> list_eq k' l'
                  | Right -> False))

(** val confs_red : Delta.t -> state0 -> ident -> substitutions0 -> bool **)

let confs_red d0 q x x0 =
  forallb (fun s ->
    match s x with
      | Some y ->
          (match y with
             | State q' ->
                 forallb (fun f ->
                   forallb (fun conf ->
                     existsb (list_eq conf)
                       (SetList.lists (Delta.get q f d0)))
                     (SetList.lists (Delta.get q' f d0)))
                   (Delta.symbol_list q' d0)
             | _ -> False)
      | None -> False) x0

(** val closure_at_q : Delta.t -> state0 -> term -> term -> bool **)

let closure_at_q d0 q l r =
  all_reduce d0 q r (matching d0 q l)

(** val closure_rule : Delta.t -> term -> term -> bool **)

let closure_rule d0 l r =
  forallb (fun q -> closure_at_q d0 q l r) (Delta.dom d0)

(** val closure_at_q' : Delta.t -> state0 -> term -> ident -> bool **)

let closure_at_q' d0 q l x =
  confs_red d0 q x (matching d0 q l)

(** val closure_rule' : Delta.t -> term -> ident -> bool **)

let closure_rule' d0 l x =
  forallb (fun q -> closure_at_q' d0 q l x) (Delta.dom d0)

(** val closure_by_llrule : Delta.t -> term -> term -> bool **)

let closure_by_llrule d0 l r = match r with
  | Var x -> closure_rule' d0 l x
  | _ -> closure_rule d0 l r

(** val closure_by_l_rules : Delta.t -> l_rules -> bool **)

let closure_by_l_rules d0 x =
  forallb (fun lr -> let Lrule (l, r) = lr in closure_by_llrule d0 l r) x

(** val postFix : automaton -> automaton -> l_rules -> bool **)

let postFix a b r =
  match inclusion_dec a b with
    | True -> closure_by_l_rules b.d r
    | False -> False

