open Ast
open Coq_checker
open Coq_interface

let symbol env x = SMap.mem x env.symbols
  
let var env x = SMap.mem x env.vars

let arity env x =
  try 
    let _, ar = SMap.find x env.symbols
    in
      ar
  with
      Not_found -> 0

let checkSymTbl st =
  SMap.iter
    ( fun k (id, _) ->
	try
	  let _, ar = 
	    SMap.find k st.symbols
	  in
	    if ar = 0 then
	      Errors.warn id
		(Format.sprintf
		   "Warning : Variable %s is already as a constant" k
		)
	with
	    Not_found -> ()
    )
    st.vars

let rec chk_context rhs e = function
  | T(id, tl) ->
      if (symbol e id.raw) then
	let ar = arity e id.raw 
	and len = List.length tl in
	  if (ar = len) then
	    List.iter (chk_context rhs e) tl
	  else
	    Errors.error Errors.objCode id
	      (Format.sprintf "Constructor %s expects %i args but here it is
				used with %i args" id.raw ar len) 
      else if (var e id.raw) then
	(
	  if (List.length tl <> 0) then
	    Errors.error Errors.objCode id
	      (Format.sprintf "Variable %s does not expect any arg" id.raw)
	)
      else
	Errors.error Errors.objCode id
	  (Format.sprintf "%s is not declared" id.raw)

  | VarSpe id when (not rhs) ->
      Errors.error Errors.objCode id
	(Format.sprintf "Special %s can be used only in left hand-side of a rule"
	   id.raw)

  | VarSpe _ -> ()
      
      
let check_trs_list e =
  let chk_trs (_, rules) =
    let chk_rule = function 
      | Simple (l, r) | Guarded (l, r, _) -> 
	  chk_context false e l;
	  chk_context true e r
    in
      List.iter chk_rule rules
  in
    List.iter chk_trs
(*
let check_delta e d =
  d.fold (fun _ f q lq ->
	    if var e x || not symb e x then
	      Errors.error 
		Format.eprintf "Not enough automata to check inclusion\n@?";
	      exit Errors.objCode
	    
*)       
let load_a_b = function
  | Auto (ida, a)::Auto(idb, b)::_ ->
      (ida.raw, {d=build_delta a; qf=extract_f_states a}),
      (idb.raw, {d=build_delta b; qf=extract_f_states b})
  | _ -> 
      Format.eprintf "Not enough automata to check inclusion\n@?";
      exit Errors.objCode


let load_r env = function
  | ((id, rules) as t):: _ -> 
      (id.raw, build_trs (var env) t)
  | _ -> 
      Format.eprintf "Not enough TRS to check closure\n@?";
      exit Errors.objCode

let read_file = fun path ->
  let buffer = 
    try
      Lexing.from_channel
	(if path = "" then stdin else open_in path)
    with
      | Sys_error _ ->
	  Format.eprintf
	    "I/O error: %s: No such file or directory\n@?"
	    path;
	  exit Errors.sysCode
  in
    try
      Parser.file Lexer.get_token buffer
    with
      | Lexer.LexError s -> 
	  Errors.localisation path
	    (Lexing.lexeme_start_p buffer)(Lexing.lexeme_end_p buffer);
	  Format.eprintf "Illegal character (%s)\n@?" s;
	  exit Errors.lexCode

      | Parsing.Parse_error ->
	  Errors.localisation path
	    (Lexing.lexeme_start_p buffer)(Lexing.lexeme_end_p buffer);
	  Format.eprintf "Syntax error\n@?";
	  exit Errors.synCode


