{
  open Lexing
  open Parser

  exception LexError of string

  let newline lexbuf =
    let pos = lexbuf.lex_curr_p in
      lexbuf.lex_curr_p <-
	{ pos with pos_lnum = pos.pos_lnum + 1; pos_bol = pos.pos_cnum }
	
  (* [i] stands for the comments deepness; see below... *)
  let i = ref 0

  let kwdTbl = Hashtbl.create 20
  let _ = 
    Hashtbl.add kwdTbl "Ops"	OPS;
    Hashtbl.add kwdTbl "Vars"	VARS;
    Hashtbl.add kwdTbl "TRS"	TRS;
    Hashtbl.add kwdTbl "Set"	SET;
    Hashtbl.add kwdTbl "Const"	CONST;
    Hashtbl.add kwdTbl "Prior"	PRIOR;
    Hashtbl.add kwdTbl "States"	STATES;
    Hashtbl.add kwdTbl "Rules"	RULES;
    Hashtbl.add kwdTbl "if"	IF;
    Hashtbl.add kwdTbl "equal"	EQUAL;
    Hashtbl.add kwdTbl "Patterns"	PATTERN;
    Hashtbl.add kwdTbl "SubPatterns"	SUBPATTERN;
    Hashtbl.add kwdTbl "DotPatterns"	DOTPATTERN;
    Hashtbl.add kwdTbl "Automaton"	AUTOMATON;
    Hashtbl.add kwdTbl "Transitions"	TRANSITIONS;
    Hashtbl.add kwdTbl "Equations"	EQUATIONS

  let keyWord str =
    try
      Hashtbl.find kwdTbl str
    with Not_found -> ID str

}
let space	= [' ' '\t']
let letter	= ['a'-'z' 'A'-'Z']
let digit	= ['0'-'9']
let ident	= letter (letter | digit | '_' | '-')*
let integer	= digit+
(* let comline	= '%' _* '\n' *)


rule get_token = parse
(* Spaces *)
  | space+			{ get_token lexbuf }
  | '\n' | '\r'        		{ newline lexbuf; get_token lexbuf }
(* Comments *)
  | "/*"			{ i := 1; comment lexbuf }
  | '%' 	 		{ comline lexbuf }
(* Signs *)
  | "!"				{ NOT }
  | "->"	       		{ GIVES }
  | "<->"			{ JOIN }
  | "/\\"			{ AND }
  | '['				{ LB }
  | ']'				{ RB }
  | '('				{ LP }
  | ')'				{ RP } 
  | "=>"			{ IMP }
  | '='				{ EQ }
  | ','				{ COMMA }
  | ':'				{ COLON }
  | integer as i		{ INT (int_of_string i) }
  (* Espace de merde *)
  | "Final States"		{ FSTATES }
  | ident as id			{ keyWord id }
  | '_'				{ SPEVAR }
  | eof            		{ EOF }
  | _ as c  			{ raise (LexError (Char.escaped c)) }

(* language C comments but "a la CAML" *)
and comment = parse
  | "/*"		{ i := !i + 1; comment lexbuf }
  | "*/"		{ i := !i - 1;
			  (if !i > 0 then comment else get_token)
			    lexbuf
			}
  | '\n'		{ newline lexbuf; comment lexbuf }
  | eof			{ EOF }
  | _			{ comment lexbuf }

and comline = parse
  | '\n'		{ newline lexbuf; get_token lexbuf }
  | eof			{ EOF }
  | _                   { comline lexbuf }

{

}
