open Coq_interface
open Coq_checker

let read_args () = 
  let inclusion = ref false
  and display = ref false 
  and fixpoint = ref false
  and files = ref [] in
    Arg.parse
      [("-fp", Arg.Set fixpoint, "to perform fixpoint checking");
       ("-i", Arg.Set inclusion, "to perform inclusion checking");
       ("-d", Arg.Set display, "to display objects and exit")]
      (fun s -> files := s::!files)
      ("Usage : "^Sys.executable_name^" [-c ] [-fp ] <input files>");
    !display, !inclusion, !fixpoint, !files


let display_obj (id, trs) a b =
  Format.printf "TRS %s :\n" id;
  Coq_interface.dump_trs trs;
  Format.printf "\n";
  List.iter
    (function
       | (id, a) ->
	   (Format.printf "Automaton %s :\n" id;
	    Format.printf "Final states : ";
	    Coq_interface.dump_fstates a.qf;
	    Format.printf "Tansitions:\n";
	    Coq_interface.dump_delta a.d;
	    Format.printf "\n"
	   )
    ) [a; b]
      
let perform_inclusion (ida, a) (idb, b) =
  Format.printf "Inclusion : \n";
  Format.printf "- %s included in %s :" ida idb;
  ( match (Coq_checker.inclusion_dec a b) with
      | True -> 
	  Format.printf "\t OK \n";
	  true
      | False -> 
	  Format.printf "\t KO \n";
	  false

  )


let perform_postFix (ida, a) (idb, b) (id, trs) =
  Format.printf "Post fixpoint : \n";
  Format.printf "- %s*(%s) included in %s :" id ida idb;
  ( match (Coq_checker.postFix a b trs) with
      | True -> 
	  Format.printf "\t OK \n";
	  true
      | False -> 
	  Format.printf "\t KO \n";
	  false
  )

    
let _main_ =
  let display, inclusion, fixpoint, files = read_args () in
  let symtbl, trsList, automata =
    Extraction.read_file
      ( match files with
	  | [] -> ""
	  | f::_ -> f
      )
  in let a, b = Extraction.load_a_b automata
  in let trs = Extraction.load_r symtbl trsList in
    
    if display then 
      ( 
	display_obj trs a b;
	exit Errors.okCode
      );

    let result = 
      (
	if inclusion && not fixpoint then
	  perform_inclusion a b
	else
	  true
      ) && 
	if fixpoint then
	  perform_postFix a b trs
	else
	  true
    in
      if result then
	exit Errors.okCode
      else
	exit Errors.koCode
