%{
  open Ast
  open Lexing

  let locate_id x n =
    let p1 = Parsing.rhs_start_pos n 
    and p2 = Parsing.rhs_end_pos n in
      {raw = x;
       file = p1.pos_fname;
       line = p1.pos_lnum;
       st_pos = p1.pos_cnum - p1.pos_bol;
       end_pos = p2.pos_cnum - p2.pos_bol
      }
      

%}
%token OPS CONST VARS TRS EQUATIONS
%token AUTOMATON TRANSITIONS PRIOR STATES FSTATES
%token COLON GIVES EQ JOIN NOT AND IMP
%token LB RB LP RP COMMA
%token EQUAL
%token RULES
%token SET
%token IF
%token PATTERN SUBPATTERN DOTPATTERN
%token SUBPATTERN
%token<int> INT
%token<string> ID
%token SPEVAR
%token EOF
  
%start file
%type <Ast.t> file

%%
  file : 
  OPS ops
  constSection
  VARS vars
  trsList
  automata
  patternSection
  subPatternSection
  dotPatternSection
  approxSection
  EOF				{ (new_env $2 $5, $6, $7@[$8]) }
  ;

  id :
  | ID				{ locate_id $1 1 }
  ;

  ops : 
  | id COLON INT ops		{ SMap.add $1.raw ($1, $3) $4 }
  |				{ SMap.empty }
  ;

  vars : 
  | id vars			{ SMap.add $1.raw ($1, 0) $2 }
  |				{ SMap.empty }
  ;

  trsList :
  | TRS id rules trsList	{ ($2, $3)::$4 }
  |				{ [] }
  ;

  constSection : 
  | CONST ConstList {}
  | {}
  ;

  ConstList : 
  |  ConstElem ConstList {}
  |  ConstElem {}
  ;

  ConstElem : 
    id {}
  ;

  rules : 
  | rule rules			{ $1::$2 }
  |				{ [] }
  ;

  rule : 
  | term GIVES term IF conditions	{ Guarded ($1, $3, $5) }
  | term GIVES term			{ Simple ($1, $3) }
  ;

  condition : 
  | id JOIN id				{ ($1, $3) }
  ;

  conditions : 
  | condition				{ [$1] }
  | condition AND conditions		{ $1::$3 }
  ;

  term : 
  | id LP terms RP	{ T ($1, $3) }
  | id			{ T ($1, []) }
  | SPEVAR		{ VarSpe (locate_id "_" 1) }
  ;

  terms : 
  | term		{ [$1] } 
  | term COMMA terms	{ $1::$3 }
  ;


  automata :
  | automaton automata	{ $1::$2 }
  |			{ [] }
  ;

  automaton : 
  | AUTOMATON id
    STATES states FSTATES states
    priors TRANSITIONS transitions	{ Auto ($2, {states=$4; f_states=$6;
						     prior=$7; rules=$9}) }
  | SET id termSet			{ Set ($2, $3) }
  ;

  termSet : 
  | term termSet	{ $1::$2 }
  |			{ [] }
  ;
  
  states : 
  | id states		{ $1::$2 }
  |			{ [] }
  ;

  priors : 
  | PRIOR transitions	{ $2 } 
  |			{ [] }
  ;
  transitions : 
  | normalTerm GIVES id transitions	{ Simple ($1, T($3, []))::$4 }
  |					{ [] }
  ;
  normalTerm : 
  | id					{ T ($1, []) }
  | id LP normalTerms RP		{ T ($1, $3) }
  ;
  normalTerms :
  | id COMMA normalTerms		{ T($1, [])::$3 }
  | id					{ [ T($1, []) ] }
  ;


  patternSection : 
  | PATTERN patterns			{ Set(locate_id id_pattern 1, $2) }
  | SUBPATTERN patterns                 { Set(locate_id id_pattern 1, $2) }
  |					{ Set(locate_id id_pattern 1, []) }
  ;
  
  subPatternSection : 
  | SUBPATTERN patterns			{ }
  |					{ }
  ;

  dotPatternSection : 
  | DOTPATTERN patterns			{ }
  |					{ }
  ;

  patterns :
  | term				{ [$1] } 
  | term patterns			{ $1::$2 }
  ;



  approxSection : 
  | approxElem approxSection		{ $1::$2 }
  |					{ [] }
  ;

  approxElem : 
  | EQUATIONS id RULES equations	{ ($2, $4) }
  ;


  equations :
  | equation equations		{ () }
  |				{ () }
  ;

  equation :
  | term EQ term				{ () }
  | term EQ term IF conditionsEq		{ () }
  | LB term EQ term RB IMP LB simpleEqs RB	{ () }
  | LB term COMMA term RB IMP LB simpleEqs RB	{ () }
  | LB term RB IMP LB simpleEqs RB		{ () }
  ;

  simpleEqs :
  | term EQ term simpleEqs		{ () }
  |					{ () }
  ;
/*
  equations :
    equation IF conditionsEq equations	{ ($1, $3)::$4 }
  | equation equations			{ ($1, [])::$2 }
  |					{ [] }
  ;

  equation : 
  | term EQ term			{ ([], $1, $3) }
  | LB terms RB IMP term EQ term	{ ($2, $5, $7) }
  ;
*/

  conditionEq : 
  | id NOT EQ term			 { () }
  ;

  conditionsEq : 
  | conditionEq				 { () }
  | conditionEq AND conditionsEq	 { () }
  ;

%%


(* trailer-and-end *)
